import { ImageBaseUrl } from "@/configs/api";
import { createProxyUrl } from "@/utils/proxy";
import { useGLTF } from "@react-three/drei";
import React, { useEffect, useState } from "react";
import * as THREE from "three";
import { Group, Mesh } from "three";
import { DRACOLoader } from "three/examples/jsm/loaders/DRACOLoader.js";
import { GLTF, GLTFLoader } from "three/examples/jsm/loaders/GLTFLoader.js";

// Preload models
useGLTF.preload("/glbs/newpool1.glb");
useGLTF.preload("/glbs/newpool2.glb");
useGLTF.preload("/glbs/pool1floor1.glb");
useGLTF.preload("/glbs/pool2floor1.glb");

type SelectType = {
	pool: number;
	floor: number;
	ladder: number;
	waterfall: number;
	jacuzzi: number;
	lamp: number;
	showWater: boolean;
	selectedTile?: {
		id: number;
		texture: string;
		name: string;
		model_3d?: string;
	} | null;
	selectedLadder?: {
		id: number;
		texture: string;
		name: string;
		model_3d?: string;
	} | null;
	selectedLight?: {
		id: number;
		texture: string;
		name: string;
		model_3d?: string;
	} | null;
	selectedWaterfall?: {
		id: number;
		texture: string;
		name: string;
		model_3d?: string;
	} | null;
};

export default function Model({ showItem }: { showItem: SelectType }) {
	const group = React.useRef<Group>(null);
	const [currentTexture, setCurrentTexture] = useState<THREE.Texture | null>(null);
	const [isLoadingTexture, setIsLoadingTexture] = useState<boolean>(false);
	const [useFallbackMaterial, setUseFallbackMaterial] = useState<boolean>(false);
	const [textureError, setTextureError] = useState<string | null>(null);
	const [customModel, setCustomModel] = useState<THREE.Group | null>(null);
	const [isLoadingModel, setIsLoadingModel] = useState<boolean>(false);
	const [customLightModel, setCustomLightModel] = useState<THREE.Group | null>(null);
	const [isLoadingLightModel, setIsLoadingLightModel] = useState<boolean>(false);
	const [customWaterfallModel, setCustomWaterfallModel] = useState<THREE.Group | null>(null);
	const [isLoadingWaterfallModel, setIsLoadingWaterfallModel] = useState<boolean>(false);

	console.log("showItem: ", showItem);

	// Load pool models
	const { scene: pool_shape_1 } = useGLTF("/glbs/newpool1.glb");
	const { scene: pool_shape_2, nodes } = useGLTF("/glbs/newpool2.glb");

	// Load floor models
	const { scene: floor_type_1 } = useGLTF("/glbs/pool1floor1.glb");
	const { scene: floor_type_2 } = useGLTF("/glbs/pool2floor1.glb");

	// Set floor positions
	floor_type_1.position.set(0, 0.001, 0);
	floor_type_2.position.set(0, 0.001, 0);

	// Pool and floor options
	const _selectedOptions = React.useMemo(() => [
		{
			pool: pool_shape_1,
			floors: floor_type_1,
		},
		{
			pool: pool_shape_2,
			floors: floor_type_2,
		},
	], [pool_shape_1, pool_shape_2, floor_type_1, floor_type_2]);

	// Shared function to load a 3D model
	const loadModel = async (modelUrl: string, setModel: (model: THREE.Group | null) => void, setIsLoading: (loading: boolean) => void) => {
		if (!modelUrl) {
			setModel(null);
			return;
		}
		console.log("modelUrl: ", modelUrl);
		setIsLoading(true);
		try {
			const proxyUrl = createProxyUrl(modelUrl, "model");
			console.log("proxyUrl: ", proxyUrl);
			const loader = new GLTFLoader();

			// Set up DRACO loader
			const dracoLoader = new DRACOLoader();
			dracoLoader.setDecoderPath("https://www.gstatic.com/draco/versioned/decoders/1.5.6/"); // Use Google's hosted decoder
			loader.setDRACOLoader(dracoLoader);

			const gltf = await new Promise<GLTF>((resolve, reject) => {
				loader.load(
					proxyUrl,
					resolve,
					undefined,
					reject
				);
			});
			console.log("gltf: ", gltf);
			setModel(gltf.scene as THREE.Group);
		} catch (error) {
			console.error("Failed to load model:", error);
			setModel(null);
		} finally {
			setIsLoading(false);
		}
	};

	// Load texture using proxy server
	useEffect(() => {
		const loadTexture = async () => {
			setIsLoadingTexture(true);
			setTextureError(null);

			try {
				const textureUrl = ImageBaseUrl + showItem.selectedTile?.texture || "/swimming-pool/floor-type-1.png";
				const proxyUrl = createProxyUrl(textureUrl, "image");

				const textureLoader = new THREE.TextureLoader();
				textureLoader.setCrossOrigin("anonymous");

				const texture = await new Promise<THREE.Texture>((resolve, reject) => {
					textureLoader.load(
						proxyUrl,
						(texture) => {
							texture.wrapS = THREE.RepeatWrapping;
							texture.wrapT = THREE.RepeatWrapping;
							texture.repeat.set(5, 5);
							texture.colorSpace = THREE.SRGBColorSpace;
							texture.needsUpdate = true;
							resolve(texture);
						},
						undefined,
						(error) => {
							console.error("Error loading texture from proxy:", error);
							reject(error);
						}
					);
				});

				setCurrentTexture(texture);
				setUseFallbackMaterial(false);
			} catch (error) {
				console.error("Failed to load texture:", error);
				setTextureError(error instanceof Error ? error.message : "Unknown error");
				setUseFallbackMaterial(true);
			} finally {
				setIsLoadingTexture(false);
			}
		};

		if (showItem?.selectedTile?.texture) loadTexture();
	}, [showItem?.selectedTile?.texture]);

	// Load custom 3D model using proxy server
	useEffect(() => {
		if (showItem.selectedLadder?.model_3d) {
			console.log("showItem.selectedLadder?.model_3d: ", showItem.selectedLadder?.model_3d);
			const modelUrl = ImageBaseUrl + showItem.selectedLadder?.model_3d;
			loadModel(modelUrl, setCustomModel, setIsLoadingModel);
		} else {
			setCustomModel(null);
		}
	}, [showItem?.selectedLadder?.model_3d]);


	useEffect(() => {
		if (showItem.selectedLight?.model_3d) {
			console.log("showItem.selectedLight?.model_3d: ", showItem.selectedLight?.model_3d);
			const modelUrl = ImageBaseUrl + showItem.selectedLight?.model_3d;
			loadModel(modelUrl, setCustomLightModel, setIsLoadingLightModel);
		} else {
			setCustomModel(null);
		}
	}, [showItem?.selectedLight?.model_3d]);


	useEffect(() => {
		if (showItem.selectedWaterfall?.model_3d) {
			console.log("showItem.selectedWaterfall?.model_3d: ", showItem.selectedWaterfall?.model_3d);
			const modelUrl = ImageBaseUrl + showItem.selectedWaterfall?.model_3d;
			loadModel(modelUrl, setCustomWaterfallModel, setIsLoadingWaterfallModel);
		} else {
			setCustomWaterfallModel(null);
		}
	}, [showItem?.selectedWaterfall?.model_3d]);


	// Apply texture to floor meshes
	useEffect(() => {
		if (showItem.pool === 1) {
			const currentFloor = _selectedOptions[showItem.pool - 1].floors;
			currentFloor.traverse((child) => {
				if (child instanceof Mesh) {
					if (["Cube001"].includes(child.name)) { // , Cube001_1, Cube009, Cube014
						const material = child.material.clone();

						if (useFallbackMaterial) {
							// Use a solid color material as fallback
							material.map = null;
							material.color = new THREE.Color(0xcccccc); // Light gray color
							material.needsUpdate = true;
						} else if (currentTexture) {
							// Use the loaded texture
							material.map = currentTexture;
							material.needsUpdate = true;
						}

						child.material = material;
					}
				}
			});
		}

		if (showItem.pool === 2) {
			pool_shape_2.traverse((child) => {
				if (child instanceof Mesh) {
					if (["Plane033"].includes(child.name)) {
						const material = child.material.clone();

						if (useFallbackMaterial) {
							// Use a solid color material as fallback
							material.map = null;
							material.color = new THREE.Color(0xcccccc); // Light gray color 0xcccccc
							material.needsUpdate = true;
						} else if (currentTexture) {
							// Use the loaded texture
							material.map = currentTexture;
							material.needsUpdate = true;
						}

						child.material = material;
					}
				}
			});

		}
	}, [currentTexture, useFallbackMaterial, showItem.pool, _selectedOptions]);

	console.log("customModel: ", customModel);

	return (
		<group ref={group}>
			{/* Render pool shape */}
			{showItem.pool && (
				<React.Fragment>
					<primitive object={_selectedOptions[showItem.pool - 1].pool} />
				</React.Fragment>
			)}

			{/* Render floor */}
			{showItem?.selectedTile?.id && (
				<primitive object={_selectedOptions[showItem.pool - 1].floors} />
			)}

			{/* Render ladder */}
			{showItem?.selectedLadder?.id && customModel && (
				<primitive object={customModel} />
			)}

			{/* Render light */}
			{showItem?.selectedLight?.id && customLightModel && (
				<primitive object={customLightModel} />
			)}

			{/* Render waterfall */}
			{showItem?.selectedWaterfall?.id && customWaterfallModel && (
				<primitive object={customWaterfallModel} />
			)}
		</group>
	);
}